/**
 * \file navigator.cpp
 *
 * Source file for the navigator tool.
 *
 * \date 2011-01-09
 * \author redouane.boumghar@laas.fr
 * \ingroup rednav
 */

#include <rednav/navigator.hpp>

namespace jafar {
namespace rednav {

	// Constructor that does nothing but giving calls to tools
	Navigator::Navigator()
	{ // {{{
	} // }}}

	// Static map file Constructor
	Navigator::Navigator(const std::string &inputfile)
	{ //{{{
		navMap = new jafar::lgl::GeoData(inputfile);
		if (navMap != NULL)
		{
			navQuadtree = new jafar::lgl::Quadtree<double, jafar::lgl::GeoData>(navMap);
			navGraph = new jafar::lgl::NavGraph(navMap, navQuadtree);
			multiTool = new PathTool();
		}
	} //}}}

	// Tooled Constructor
	Navigator::Navigator(double* bufferedData, int xsize_, int ysize_, int zsize_, double xMapOrigin_, double yMapOrigin_, double zMapOrigin_, double pixelScale_)
	{ //{{{
		setMapOrigin(xMapOrigin_, yMapOrigin_, zMapOrigin_, pixelScale_);
		setMapSize(xsize_, ysize_, zsize_);

		navMap = new jafar::lgl::GeoData(xsize, ysize, bufferedData);
		navQuadtree = new jafar::lgl::Quadtree<double, jafar::lgl::GeoData>(navMap);
		navGraph = new jafar::lgl::NavGraph(navMap, navQuadtree);
		multiTool = new PathTool();
	} //}}}

	//-------------------------- Destructor ---------
	Navigator::~Navigator()
	{ // {{{
		delete navMap;
		delete navQuadtree;
		delete navGraph;
	} // }}}

	bool Navigator::isUp()
	{//{{{
		if (navMap == NULL || navQuadtree == NULL || navGraph == NULL)
		{
			return false;
		}
		return true;
	}//}}}

	bool Navigator::saveData(const std::string& outfile, const std::string& toutfile, const std::string& quadTreeFile)
	{/*{{{*/
		navMap->saveData(outfile);
		navMap->saveTraversability(toutfile);
		std::ofstream quadTreeStream (quadTreeFile.c_str());
		quadTreeStream << (*navQuadtree);
		quadTreeStream.close();
		return true;
	}/*}}}*/

	bool Navigator::findPath(const jafar::lgl::NavLocation& locstart, const jafar::lgl::NavLocation& locend, jafar::lgl::NavPath& navpath)
	{//{{{
		return navGraph->findPath(locstart, locend, navpath);
	}//}}}

	bool Navigator::findPathOnUtility(const jafar::lgl::NavLocation& locstart, const jafar::lgl::NavLocation& locend, jafar::lgl::NavPath& navpath)
	{//{{{
		return navGraph->findPathOnUtility(locstart, locend, navpath);
	}//}}}

	bool Navigator::updateMap(double* bufferedData, int xsize_, int ysize_)
	{//{{{
		delete navGraph;
		navQuadtree = NULL;
		setMapSize(xsize_, ysize_, zsize);
		if (navMap->setData(bufferedData, xsize, ysize))
		{
			// Recreating quadtree
			navQuadtree = new jafar::lgl::Quadtree<double, jafar::lgl::GeoData>(navMap);
			// Recreating navgraph
			navGraph = new jafar::lgl::NavGraph(navMap, navQuadtree);
			return true;
		}
		return false;
	}//}}}

	//---- set stuffs

	void Navigator::setMapOrigin(double xMapOrigin_, double yMapOrigin_, double zMapOrigin_, double pixelScale_)
	{//{{{
		xMapOrigin = xMapOrigin_;
		yMapOrigin = yMapOrigin_;
		zMapOrigin = zMapOrigin_;
		pixelScale = pixelScale_;
	}//}}}

	void Navigator::setMapSize(int  xsize_, int ysize_, int zsize_)
	{//{{{
		xsize = xsize_;
		ysize = ysize_;
		zsize = zsize_;
	}//}}}

	void Navigator::setGoal(int x, int y, int z)
	{//{{{
		goalx = x;
		goaly = y;
		goalz = z;
	}//}}}

	void Navigator::setGoal(double x, double y, double z)
	{//{{{
		goalx = x;
		goaly = y;
		goalz = z;
	}//}}}

	void Navigator::setStart(int x, int y, int z)
	{//{{{
		startx = x;
		starty = y;
		startz = z;
	}//}}}

	void Navigator::setStart(double x, double y, double z)
	{//{{{
		startx = x;
		starty = y;
		startz = z;
	}//}}}

	void Navigator::setPathCropers(double pce, double pcp)
	{//{{{
		if (pce >= 0.0)
		{
			multiTool->setPathMaxEntropy(pce);
		}

		if (pcp >= 0.0)
		{
			multiTool->setPathMaxProba(pcp);
		}
	}//}}}

	//---- get stuffs
	
	jafar::lgl::NavGraph::NavGraphContainer* Navigator::getGraph()
	{/*{{{*/
		if (navGraph != NULL)
		{
			return navGraph->getGraph();
		}
		return NULL;
	}/*}}}*/

	jafar::lgl::Decomp* Navigator::getDecomposition()
	{/*{{{*/
		if (navQuadtree != NULL)
		{
			return navQuadtree->getDecomp();
		}
		return NULL;
	}/*}}}*/

	bool Navigator::cropPath(double vent, double vproba, jafar::lgl::NavPath& navpath)
	{//{{{
		return (multiTool->cropPath(vent, vproba, navpath));
	}//}}}

	bool Navigator::applyTool(jafar::lgl::NavPath& srcPath, jafar::lgl::NavPath& outPath, PATH_TRANSFORMATION ptrans)
	{//{{{
		return (multiTool->applyTool(srcPath, outPath, ptrans));
	}//}}}
}} // end of namespaces
